package com.viettel.einvoice.service.impl;

import com.viettel.einvoice.config.properties.BaseServiceProperties;
import com.viettel.einvoice.config.properties.EInvoiceProperties;
import com.viettel.einvoice.config.properties.InvoiceServiceProperties;
import com.viettel.einvoice.dto.*;
import com.viettel.einvoice.service.InvoiceService;
import com.viettel.einvoice.service.InvoiceServiceTest;
import com.viettel.einvoice.service.object.*;
import com.viettel.einvoice.utils.DateTimeUtils;
import com.viettel.einvoice.utils.HttpRequestUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class InvoiceServiceImpl extends BaseServiceImpl implements InvoiceService {
    EInvoiceProperties eInvoiceProperties;
    private InvoiceServiceProperties properties;

    public InvoiceServiceImpl() {
        eInvoiceProperties = EInvoiceProperties.instance().readProperties();
        this.properties = eInvoiceProperties.getInvoiceService();
        this.setUsername(eInvoiceProperties.getUsername());
        this.setPassword(eInvoiceProperties.getPassword());
    }

    public InvoiceCreateDTO create(Map<String, String> paramsUrl, InvoiceCreateObject invoiceCreateObject) {
        return create(properties.getCreate(), paramsUrl, invoiceCreateObject);
    }

    @Override
    public InvoiceCreateDTO createBatch(Map<String, String> paramsUrl, List<InvoiceCreateObject> invoiceCreateObjects) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties createProperties = properties.getCreateBatch();

        InvoiceCreateBatchObject invoiceCreateBatchObject = new InvoiceCreateBatchObject();
        invoiceCreateBatchObject.setCommonInvoiceInputs(invoiceCreateObjects);

        String url = this.setParamsToUrl(paramsUrl, this.getHost(eInvoiceProperties) + createProperties.getApi());

        return HttpRequestUtils.instances(true)
                .url(url)
                .method(createProperties.getMethod())
                .headers(headers)
                .body(invoiceCreateBatchObject)
                .requestObject(InvoiceCreateDTO.class);
    }

    @Override
    public InvoiceCreateDTO createWithUsbTokenGetHash(Map<String, String> paramsUrl, InvoiceCreateObject invoiceCreateObjects) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties getHashProperties = properties.getCreateUsbTokenGetHash();

        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + getHashProperties.getApi());

        return HttpRequestUtils.instances(true)
                .url(url)
                .method(getHashProperties.getMethod())
                .headers(headers)
                .body(invoiceCreateObjects)
                .requestObject(InvoiceCreateDTO.class);
    }

    @Override
    public InvoiceCreateDTO createWithUsbTokenInsertHash(InvoiceHashObject invoiceHashObject) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties insertHashProperties = properties.getCreateUsbTokenInsertHash();

        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + insertHashProperties.getApi());

        return HttpRequestUtils.instances(true)
                .url(url)
                .method(insertHashProperties.getMethod())
                .headers(headers)
                .body(invoiceHashObject)
                .requestObject(InvoiceCreateDTO.class);
    }

    public InvoiceCreateDTO createDraft(Map<String, String> paramsUrl, InvoiceCreateObject invoiceCreateObject) {
        return create(properties.getCreateDraft(), paramsUrl, invoiceCreateObject);
    }

    private InvoiceCreateDTO create(BaseServiceProperties createProperties, Map<String, String> paramsUrl, InvoiceCreateObject invoiceCreateObject) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());

        String url = this.setParamsToUrl(paramsUrl, this.getHost(eInvoiceProperties) + createProperties.getApi());

        return HttpRequestUtils.instances(true)
                .url(url)
                .method(createProperties.getMethod())
                .headers(headers)
                .body(invoiceCreateObject)
                .requestObject(InvoiceCreateDTO.class);
    }

    @Override
    public byte[] createFileExchange(InvoiceFileObject invoiceFileObject) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties fileExchangeProperties = properties.getCreateExchangeFile();

        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + fileExchangeProperties.getApi());
        InvoiceFileDTO invoiceFileDTO = HttpRequestUtils.instances(true)
                .url(url)
                .method(fileExchangeProperties.getMethod())
                .headers(headers)
                .body(invoiceFileObject)
                .requestObject(InvoiceFileDTO.class);

        return invoiceFileDTO.getFileToBytes();
    }

    @Override
    public CommonDTO cancel(InvoiceCancelObject invoiceCancelObject) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties cancelProperties = properties.getCancel();

        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + cancelProperties.getApi());
        return HttpRequestUtils.instances(true)
                .url(url)
                .method(cancelProperties.getMethod())
                .headers(headers)
                .body(invoiceCancelObject)
                .requestObject(CommonDTO.class);
    }

    @Override
    public InvoiceCreateDTO getCustomFields(Map<String, String> params) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties getFieldsProperties = properties.getGetCustomField();

        String url = this.setParamsToUrl(params, this.getHost(eInvoiceProperties) + getFieldsProperties.getApi());
        return HttpRequestUtils.instances(true)
                .url(url)
                .method(getFieldsProperties.getMethod())
                .headers(headers)
                .requestObject(InvoiceCreateDTO.class);
    }

    @Override
    public CommonDTO updateTaxDeclaration(String supplierTaxCode, Date issueDate) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties updateTaxProperties = properties.getGetCustomField();

        InvoiceFileObject invoiceFileObject = new InvoiceFileObject();
        invoiceFileObject.setSupplierTaxCode(supplierTaxCode);
        invoiceFileObject.setStrIssueDate(DateTimeUtils.convertDateToString(issueDate, "dd/MM/yyyy"));

        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + updateTaxProperties.getApi());
        return HttpRequestUtils.instances(true)
                .url(url)
                .method(updateTaxProperties.getMethod())
                .headers(headers)
                .body(invoiceFileObject)
                .requestObject(CommonDTO.class);
    }
}
