package com.viettel.einvoice.service.impl;

import com.viettel.einvoice.config.properties.BaseServiceProperties;
import com.viettel.einvoice.config.properties.EInvoiceProperties;
import com.viettel.einvoice.config.properties.InvoiceUtilsServiceProperties;
import com.viettel.einvoice.dto.*;
import com.viettel.einvoice.service.InvoiceUtilsServiceTest;
import com.viettel.einvoice.service.object.*;
import com.viettel.einvoice.utils.HttpRequestUtils;

import java.util.*;

public class InvoiceUtilsServiceImpl extends BaseServiceImpl implements InvoiceUtilsServiceTest {
    EInvoiceProperties eInvoiceProperties;
    private InvoiceUtilsServiceProperties properties;

    public InvoiceUtilsServiceImpl() {
        eInvoiceProperties = EInvoiceProperties.instance().readProperties();
        this.properties = eInvoiceProperties.getInvoiceUtilsService();
        this.setUsername(eInvoiceProperties.getUsername());
        this.setPassword(eInvoiceProperties.getPassword());
    }

    @Override
    public InvoiceSearchDTO get(Map<String, String> params, InvoiceSearchObject invoiceSearchObject) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties getProperties = properties.getGetInvoice();

        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + getProperties.getApi());
        return HttpRequestUtils.instances(true)
                .url(url)
                .method(getProperties.getMethod())
                .headers(headers)
                .body(invoiceSearchObject)
                .requestObject(InvoiceSearchDTO.class);
    }

    @Override
    public byte[] getFileRepresentation(InvoiceFileObject invoiceFileObject) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties fileRepresentationProperties = properties.getGetRepresentationFile();

        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + fileRepresentationProperties.getApi());
        InvoiceFileDTO invoiceFileDTO = HttpRequestUtils.instances(true)
                .url(url)
                .method(fileRepresentationProperties.getMethod())
                .headers(headers)
                .body(invoiceFileObject)
                .requestObject(InvoiceFileDTO.class);

        return invoiceFileDTO.getFileToBytes();
    }

    @Override
    public byte[] getFilePortal(InvoiceFileObject invoiceFileObject) {
        BaseServiceProperties filePortalProperties = properties.getGetPortalFile();
        return getFilePortal(filePortalProperties, invoiceFileObject);
    }

    @Override
    public byte[] postFilePortal(InvoiceFileObject invoiceFileObject) {
        BaseServiceProperties filePortalProperties = properties.getPostPortalFile();
        return getFilePortal(filePortalProperties, invoiceFileObject);
    }

    private byte[] getFilePortal(BaseServiceProperties properties, InvoiceFileObject invoiceFileObject) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + properties.getApi());
        InvoiceFileDTO invoiceFileDTO = HttpRequestUtils.instances(true)
                .url(url)
                .method(properties.getMethod())
                .headers(headers)
                .body(invoiceFileObject)
                .requestObject(InvoiceFileDTO.class);

        return invoiceFileDTO.getFileToBytes();
    }

    @Override
    public CommonDTO getProvidesStatusUsingInvoice(String supplierTaxCode, String pattern, String serial) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties statusProperties = properties.getGetProvicesStatusUsing();
        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + statusProperties.getApi());

        InvoiceObject invoiceObject = new InvoiceObject().supplierTaxCode(supplierTaxCode).pattern(pattern).serial(serial);

        return HttpRequestUtils.instances(true)
                .url(url)
                .method(statusProperties.getMethod())
                .headers(headers)
                .body(invoiceObject)
                .requestObject(CommonDTO.class);
    }

    @Override
    public CommonDTO sendHtmlMailProcess(String supplierTaxCode, String[] lstTransactionUuid) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", getAuthentication());
        BaseServiceProperties emailProperties = properties.getSendHtmlEmailProcess();
        String url = this.setParamsToUrl(null, this.getHost(eInvoiceProperties) + emailProperties.getApi());

        SendEmailObject sendEmailObject = new SendEmailObject();
        sendEmailObject.setSupplierTaxCode(supplierTaxCode);
        sendEmailObject.setLstTransactionUuid(lstTransactionUuid);

        return HttpRequestUtils.instances(true)
                .url(url)
                .method(emailProperties.getMethod())
                .headers(headers)
                .body(sendEmailObject)
                .requestObject(CommonDTO.class);
    }
}
