package com.viettel.einvoice.utils;

import com.google.gson.Gson;
import com.sun.istack.internal.NotNull;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLContexts;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.client.ClientHttpRequestFactory;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.util.ResourceUtils;
import org.springframework.web.client.RestTemplate;
import sun.misc.IOUtils;

import javax.net.ssl.SSLContext;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.*;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Map;
import java.util.Properties;

public class HttpRequestUtils {

    private final Logger log = LoggerFactory.getLogger(HttpRequestUtils.class);

    private RestTemplate restTemplate;
    private Object body;

    @NotNull
    private String url;

    @NotNull
    private String method;
    private HttpHeaders httpHeaders;

    public static HttpRequestUtils instances(boolean hasSSL) {
        HttpRequestUtils httpRequestUtils = new HttpRequestUtils();
        if (hasSSL) {
            httpRequestUtils.restTemplate = new RestTemplate(httpRequestUtils.httpRequestFactory());
        } else {
            httpRequestUtils.restTemplate = new RestTemplate();
        }
        httpRequestUtils.httpHeaders = new HttpHeaders();
        httpRequestUtils.httpHeaders.setContentType(MediaType.APPLICATION_JSON_UTF8);
        return httpRequestUtils;
    }

    public HttpRequestUtils url(String url) {
        this.url = url;
        return this;
    }

    public HttpRequestUtils method(String method) {
        this.method = method;
        return this;
    }

    public HttpRequestUtils body(Object body) {
        this.body = body;
        return this;
    }

    public HttpRequestUtils headers(Map<String, String> headers) {
        for (String key : headers.keySet()) {
            this.httpHeaders.add(key, headers.get(key));
        }
        return this;
    }

    public HttpRequestUtils headers(HttpHeaders httpHeaders) {
        this.httpHeaders = httpHeaders;
        return this;
    }

    public <T> T requestObject(Class<T> clazzType) {
        if (HttpMethod.GET.matches(this.method)) {
            return this.get(clazzType);
        } else if (HttpMethod.POST.matches(this.method)) {
            return this.post(clazzType, this.body);
        } else if (HttpMethod.PUT.matches(this.method)) {
            return this.put(clazzType);
        }
        return null;
    }

    private <T> T get(Class<T> clazzType) {
        return restTemplate.getForObject(this.url, clazzType);
    }

    private <T> T post(Class<T> clazzType, Object body) {
        HttpEntity<String> entity = new HttpEntity<>(new Gson().toJson(body), this.httpHeaders);
        return restTemplate.postForObject(this.url, entity, clazzType);
    }

    private <T> T put(Class<T> clazzType) {
        return restTemplate.getForObject(this.url, clazzType);
    }

    private HttpComponentsClientHttpRequestFactory httpRequestFactory() {
        HttpComponentsClientHttpRequestFactory requestFactory =
                new HttpComponentsClientHttpRequestFactory();
        try {
            SSLContext sslContext = this.getSSLContext();
            SSLConnectionSocketFactory csf = new SSLConnectionSocketFactory(sslContext, new String[]{"TLSv1", "TLSv1.1", "TLSv1.2"}, null, new NullHostnameVerifier());
            RequestConfig requestConfig = RequestConfig.custom()
                    .setConnectTimeout(60000)
                    .setSocketTimeout(60000).build();
            CloseableHttpClient httpClient = HttpClients.custom()
                    .setSSLSocketFactory(csf)
                    .setDefaultRequestConfig(requestConfig)
                    .setConnectionManagerShared(true)
                    .disableAutomaticRetries()
                    .build();

            requestFactory.setHttpClient(httpClient);
        } catch (Exception e) {
            log.error(e.getLocalizedMessage());
        } finally {
            return requestFactory;
        }
    }

    private SSLContext getSSLContext() throws KeyStoreException,
            NoSuchAlgorithmException, CertificateException, IOException, KeyManagementException, UnrecoverableKeyException {
        Properties properties = new Properties();
        properties.load(this.getClass().getClassLoader().getResourceAsStream("config.properties"));
        String trustoreFile = properties.getProperty("trust_store_client_file");
        String trustStorePassword = PasswordUtils.viettelDecrypt(properties.getProperty("trust_store_client_pass"));
        String keyStoreFile = properties.getProperty("keystore_store_client_file");
        String keyPassword = PasswordUtils.viettelDecrypt(properties.getProperty("keystore_store_client_pass"));

        KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
        FileInputStream instream = new FileInputStream(new File(trustoreFile));
        try {
            trustStore.load(instream, trustStorePassword.toCharArray());
        } finally {
            instream.close();
        }
        KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
        FileInputStream instreams2 = new FileInputStream(new File(keyStoreFile));
        try {
            keyStore.load(instreams2, keyPassword.toCharArray());
        } finally {
            instream.close();
        }
        return SSLContexts.custom()
                .loadTrustMaterial(trustStore)
                .loadKeyMaterial(keyStore, keyPassword.toCharArray())//link loadbalancer khong can phan cau hinh keyStore nay
                .build();
    }
}
