package com.viettel.einvoice.utils;

import com.viettel.security.PassTranformer;
import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.MessageDigest;
import java.util.MissingResourceException;
import java.util.Random;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class PasswordUtils {

    private static final Logger logger = LoggerFactory.getLogger(PasswordUtils.class);
    public final static ResourceBundle rb = ResourceBundle.getBundle("securitykey");
    private static String securitykey = "";

    static {
        try {
            securitykey = viettelDecryptDefault(rb.getString("securitykey"));
        } catch (MissingResourceException e) {
            logger.error("khong tim thay securitykey", e);
        } catch (Exception ex) {
            logger.error("Error", ex);
        }
    }

    /**
     * easypass
     */
    protected final static String[] easyPass = {"123", "963", "852", "741", "789", "147", "258", "369", "abc", "456"};

    /**
     * blacklist mat khau khong duoc phep
     */
//    protected final static String[] blackList = {"qwerty", "password", "passw0rd", "abc123", "iloveyou", "viettel@123", "admin@123", "123qwea@", "abc123", "abc@123",
//        "password@123", "qwerty@123", "vtt@2014", "123@123", "123123", "696969"};
    /*
     # Start of group
     (?=.*\d)	#   must contains one digit from 0-9
     (?=.*[a-z])	#   must contains one lowercase characters
     (?=.*[A-Z])	#   must contains one uppercase characters
     (?=.*[@#$%])#   must contains one special symbols in the list "@#$%"
     .		#   match anything with previous condition checking
     {8,20}	#   length at least 8 characters and maximum of 20
     )		# End of group
     */
    private static final String PASSWORD_PATTERN
            = "((?=.*\\d)"
            + "(?=.*[a-z])"
            + "(?=.*[A-Z])"
            + "(?=.*[@#$%~!%^&()])"
            + ".{8,20})";
    private static final Pattern PASSWORD_PATTERN_OBJ = Pattern.compile(PASSWORD_PATTERN);
    private static Matcher matcher;
    //public static final String DEFAULT_PASSWORD = "123456a@";
    private static final String VALID_CHARACTER = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789@#$%~!%^&()";
    private static final String UPPER_CASE_CHAR = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    private static final String LOWER_CASE_CHAR = "abcdefghijklmnopqrstuvwxyz";
    private static final String NUMBER = "0123456789";
    private static final String SPECIAL_STRING = "@#$%~!%^&()";
    private static final String SALT_DEVIDER = "#";

//    /**
//     * Validate password with regular expression
//     *
//     * @param password password for validation
//     * @return true valid password, false invalid password
//     */
//    public static boolean validateStrongPassword(final String password) {
////        matcher = pattern.matcher(password).m;
////        return matcher.matches();
//        return PASSWORD_PATTERN_OBJ.matcher(password).matches() && !isInBlackList(password);
//    }

//    public static boolean isInBlackList(String password) {
//        if (StringUtils.isNullOrEmpty(password)) {
//            return true;
//        }
//        List<ParamBO> blackList = MemoryDataLoader.getListParamBO("PASSWORD_BLACK_LIST");
//        if (!DataUtil.isNullOrEmpty(blackList)) {
//            for (ParamBO black : blackList) {
//                if (!StringUtils.isNullOrEmpty(black.getCode()) && black.getCode().equals(password)) {
//                    return true;
//                }
//            }
//        }
//        return false;
//    }

    public static String genPassAuto() {
        String password = generatePswd(8, 20);
        //return encryptPasswordBySHA256(password);
        return password;
    }

    private static String generatePswd(int minlen, int maxlen) {
        Random rand = new Random();
        int passwdlen = rand.nextInt((maxlen - minlen) + 1) + minlen;
        char[] password = new char[passwdlen];

        password[0] = UPPER_CASE_CHAR.charAt(rand.nextInt(UPPER_CASE_CHAR.length()));
        password[1] = LOWER_CASE_CHAR.charAt(rand.nextInt(LOWER_CASE_CHAR.length()));
        password[2] = NUMBER.charAt(rand.nextInt(NUMBER.length()));
        password[3] = SPECIAL_STRING.charAt(rand.nextInt(SPECIAL_STRING.length()));
        int validateCharLen = VALID_CHARACTER.length();
        for (int i = 4; i < passwdlen; i++) {
            password[i] = VALID_CHARACTER.charAt(rand.nextInt(validateCharLen));
        }
        return String.valueOf(password);
    }

    /**
     * <p>
     * Encrypt password using SHA-256 algorithm
     *
     * @param password
     * @return
     */
    private static String encryptPasswordBySHA256(String password) {
        MessageDigest md = null;
        try {
            md = MessageDigest.getInstance("SHA-256");
        } catch (Exception e) {
            logger.error("Exception", e);
        }
        if (md == null) {
            return "";
        }
        try {
            md.update(password.getBytes("UTF-8"));
        } catch (Exception e) {
            logger.error("Exception", e);
        }
        byte raw[] = md.digest();
        return Base64.encodeBase64String(raw);

    }

    /**
     * <p>
     * Append the password with a salt</p>
     *
     * @param password   The real password
     * @param saltLength The length of the salt appending with the password
     * @return
     */
    private static String encryptPassword(String password, int saltLength) {
        String salt = createSalt(saltLength);
        String input = password + salt;
        String hash = encryptPasswordBySHA256(input);
        return hash + SALT_DEVIDER + salt;
    }

    public static String encryptPassword(String password) {
        Random rand = new Random();
        int saltLength = rand.nextInt(13) + 8;// 8 <= saltLength <= 20
        return encryptPassword(password, saltLength);
    }

    /**
     * <p>
     * Check the password is true or not </p>
     *
     * @param inPassword       The pass a user enters
     * @param encrytedPassword
     * @return
     */
    public static boolean checkPassword(String inPassword, String encrytedPassword) {
        if (StringUtils.isNullOrEmpty(inPassword) || StringUtils.isNullOrEmpty(encrytedPassword)) {
            return false;
        }

        int index = encrytedPassword.lastIndexOf(SALT_DEVIDER);
        if (index == -1) {
            return false;
        }

        String password = encrytedPassword.substring(0, index);
        String salt = encrytedPassword.substring(index + SALT_DEVIDER.length(), encrytedPassword.length());
        return encryptPasswordBySHA256(inPassword + salt).equals(password);

    }

    /**
     * <p>
     * Create a String salt</p>
     *
     * @param length The Length of the salt will be created
     * @return A salt string
     */
    private static String createSalt(int length) {
        char[] salt = new char[length];
        int validateWordSize = VALID_CHARACTER.length();
        Random rand = new Random();
        do {
            for (int i = 0; i < length; i++) {
                int pos = rand.nextInt(validateWordSize);
                salt[i] = VALID_CHARACTER.charAt(pos);
            }
        } while (String.valueOf(salt).contains(SALT_DEVIDER));

        return String.valueOf(salt);
    }

    public static String encryptPassHibernate(String pass) {
        return viettelEncrypt(pass);
    }

    public static String decryptPasswordHibernate(String enscryptPass) {
        return viettelDecrypt(enscryptPass);
    }

    public static String viettelEncrypt(String data) {
        try {
            PassTranformer.setInputKey(securitykey);
            return PassTranformer.encrypt(data);
        } catch (Exception ex) {
            logger.error("", ex);
            return "";
        }
    }

    public static String viettelDecrypt(String data) {
        try {
            PassTranformer.setInputKey(securitykey);
            return PassTranformer.decrypt(data);
        } catch (Exception ex) {
            logger.error("", ex);
            return "";
        }
    }

    private static String viettelEncryptDefault(String data) {
        try {
            PassTranformer.setInputKey("");
            return PassTranformer.encrypt(data);
        } catch (Exception ex) {
            //System.out.println(ex);
            logger.error("error ", ex);
            return "";
        }
    }

    private static String viettelDecryptDefault(String data) {
        try {
            PassTranformer.setInputKey("");
            return PassTranformer.decrypt(data);
        } catch (Exception ex) {
            //System.out.println(ex);
            logger.error("Exception", ex);
            return "";
        }
    }
}
