package com.viettel.einvoice.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.Normalizer;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public final class StringUtils {

    private static final Logger logger = LoggerFactory.getLogger(StringUtils.class);

    /**
     * alphabeUpCaseNumber.
     */
    private static final String ALPHABET_UPPER_CASE_NUMBER = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    //private static final String  invoiceNoFormat = "(^[F|B|f|b][A-Za-z0-9]{3}\\-[\\d]{8})";
//    static final Pattern INVOICE_NO_FORMAT_PATTERN = Pattern.compile("(^[F|B|f|b][A-Za-z0-9]{3}\\-[\\d]{8})");
    static final Pattern IP_ADDRESS_PATTERN = Pattern.compile("^(?:[0-9\\*]{1,3}\\.){3}[0-9\\*]{1,3}$");
    private static final String EMAIL_PATTERN
            = "^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
            + "[A-Za-z0-9\\-]+(\\.[A-Za-z0-9\\-]+)*(\\.[A-Za-z]{2,})$";
    /**
     * INVOICE_MAX_LENGTH
     */
    private static final int INVOICE_MAX_LENGTH = 7;
    /**
     * ZERO.
     */
    private static final String ZERO = "0";

    /**
     * Regular expression to check the word have special character or not
     */
    private static final String SPECIAL_CHARACTER_PATTERN = "^[^\\`\\~\\!\\@\\#\\$\\%\\^\\&\\*\\(\\)\\=\\+\\[\\{\\]\\}\\\\\\|\\;\\:\\'\\\"\\,\\<\\.\\>\\/\\? ]+$";

    /**
     * Creates a new instance of StringUtils
     */
    private static final String VALID_CHARACTER = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    public static final String REGEX_PRODUCT_PRICE = "^[0-9\\,\\.E]+$";

    private StringUtils() {
    }

    public static boolean isAlpha(String name) {
        char[] chars = name.toCharArray();

        for (char c : chars) {
            if (!Character.isLetter(c)) {
                return false;
            }
        }

        return true;
    }

//    /*VT-0066 2 ky tu dau cua dai nam trong SERIAL_CHARACTERS START*/
//    public static boolean checkSerialCharacters(String value) {
//        char[] chars = value.toCharArray();
//        int isBeark = 0;
//        for (char c : chars) {
//            for (char d : Constants.getSERIAL_CHARACTERS()) {
//                if (d == c) {
//                    isBeark++;
//                }
//            }
//        }
//        if (isBeark != 2) {
//            return false;
//        }
//        return true;
//    }

    /*VT-0066 2 ky tu dau cua dai nam trong SERIAL_CHARACTERS END*/

    /**
     * method compare two string
     *
     * @param str1 String
     * @param str2 String
     * @return boolean
     */
    public static boolean compareString(String str1, String str2) {
        if (str1 == null) {
            str1 = "";
        }
        if (str2 == null) {
            str2 = "";
        }
        return str1.equals(str2);
    }

    public static boolean checkCountryCode(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }

        String pattern = "^[0-9-]+$";
        return str.matches(pattern);
    }

    /**
     * method convert long to string
     *
     * @param lng Long
     * @return String
     * @throws java.lang.Exception
     */
    public static String convertFromLongToString(Long lng) throws Exception {
        try {
            return Long.toString(lng);
        } catch (Exception ex) {
//            logger.error("loi xay ra khi convert so", ex);
            throw ex;
        }
    }

    /*
     *  @todo: convert from Long array to String array
     */
    public static String[] convertFromLongToString(Long[] arrLong) throws Exception {
        String[] arrResult = new String[arrLong.length];
        try {
            int length = arrLong.length;
            for (int i = 0; i < length; i++) {
                arrResult[i] = convertFromLongToString(arrLong[i]);
            }
            return arrResult;
        } catch (Exception ex) {
//            logger.error("loi xay ra khi convert so", ex);
            throw ex;
        }
    }

    /*
     *  @todo: convert from String array to Long array
     */
    public static long[] convertFromStringToLong(String[] arrStr) throws Exception {
        long[] arrResult = new long[arrStr.length];
        try {
            int length = arrStr.length;
            for (int i = 0; i < length; i++) {
                arrResult[i] = Long.parseLong(arrStr[i]);
            }
            return arrResult;
        } catch (Exception ex) {
//            logger.error("loi xay ra khi convert so", ex);
            throw ex;
        }
    }

    /*
     *  @todo: convert from String value to Long value
     */
    public static long convertFromStringToLong(String value) throws Exception {
        try {
            return Long.parseLong(value);
        } catch (Exception ex) {
//            logger.error("loi xay ra khi convert so", ex);
            throw ex;
        }
    }


    /*
     * Check String that containt only AlphabeUpCase and Number
     * Return True if String was valid, false if String was not valid
     */
    public static boolean checkAlphabeUpCaseNumber(String value) {
        boolean result = true;
        int length = value.length();
        for (int i = 0; i < length; i++) {
            String temp = value.substring(i, i + 1);
            if (!ALPHABET_UPPER_CASE_NUMBER.contains(temp)) {
                result = false;
                return result;
            }
        }
        return result;
    }

    public static String renameVietnameseToEnglish(String input) {
        if (isNullOrEmpty(input)) {
            return input;
        }
        String nfdNormalizedString = Normalizer.normalize(input.toLowerCase(), Normalizer.Form.NFD);
        Pattern pattern = Pattern.compile("\\p{InCombiningDiacriticalMarks}+");
        return pattern.matcher(nfdNormalizedString).replaceAll("").replace('đ', 'd').replaceAll(" +", " ");
    }

    public static String removeSignInVietnamese(String input) {
        if (isNullOrEmpty(input)) {
            return input;
        }
        String file = input;
        file = file.replace('à', 'a');
        file = file.replace('á', 'a');
        file = file.replace('ả', 'a');
        file = file.replace('ã', 'a');
        file = file.replace('ạ', 'a');
        file = file.replace('ă', 'ă');
        file = file.replace('ằ', 'ă');
        file = file.replace('ắ', 'ă');
        file = file.replace('ẳ', 'ă');
        file = file.replace('ẵ', 'ă');
        file = file.replace('ặ', 'ă');
        file = file.replace('â', 'â');
        file = file.replace('ầ', 'â');
        file = file.replace('ấ', 'â');
        file = file.replace('ẩ', 'â');
        file = file.replace('ẫ', 'â');
        file = file.replace('ậ', 'â');
        file = file.replace('À', 'a');
        file = file.replace('Á', 'a');
        file = file.replace('Ả', 'a');
        file = file.replace('Ã', 'a');
        file = file.replace('Ạ', 'a');
        file = file.replace('Ă', 'ă');
        file = file.replace('Ằ', 'ă');
        file = file.replace('Ắ', 'ă');
        file = file.replace('Ẳ', 'ă');
        file = file.replace('Ẵ', 'ă');
        file = file.replace('Ặ', 'ă');
        file = file.replace('Â', 'â');
        file = file.replace('Ầ', 'â');
        file = file.replace('Ấ', 'â');
        file = file.replace('Ẩ', 'â');
        file = file.replace('Ẫ', 'â');
        file = file.replace('Ậ', 'â');
        file = file.replace('đ', 'đ');
        file = file.replace('Đ', 'Đ');
        file = file.replace('è', 'e');
        file = file.replace('é', 'e');
        file = file.replace('ẻ', 'e');
        file = file.replace('ẽ', 'e');
        file = file.replace('ẹ', 'e');
        file = file.replace('ê', 'ê');
        file = file.replace('ề', 'ê');
        file = file.replace('ế', 'ê');
        file = file.replace('ể', 'ê');
        file = file.replace('ễ', 'ê');
        file = file.replace('ệ', 'ê');
        file = file.replace('È', 'e');
        file = file.replace('É', 'e');
        file = file.replace('Ẻ', 'e');
        file = file.replace('Ẽ', 'e');
        file = file.replace('Ẹ', 'e');
        file = file.replace('Ê', 'ê');
        file = file.replace('Ề', 'ê');
        file = file.replace('Ế', 'ê');
        file = file.replace('Ể', 'ê');
        file = file.replace('Ễ', 'ê');
        file = file.replace('Ệ', 'ê');
        file = file.replace('ì', 'i');
        file = file.replace('í', 'i');
        file = file.replace('ỉ', 'i');
        file = file.replace('ĩ', 'i');
        file = file.replace('ị', 'i');
        file = file.replace('Ì', 'i');
        file = file.replace('Í', 'i');
        file = file.replace('Ỉ', 'i');
        file = file.replace('Ĩ', 'i');
        file = file.replace('Ị', 'i');
        file = file.replace('ò', 'o');
        file = file.replace('ó', 'o');
        file = file.replace('ỏ', 'o');
        file = file.replace('õ', 'o');
        file = file.replace('ọ', 'o');
        file = file.replace('ô', 'ô');
        file = file.replace('ồ', 'ô');
        file = file.replace('ố', 'ô');
        file = file.replace('ổ', 'ô');
        file = file.replace('ỗ', 'ô');
        file = file.replace('ộ', 'ô');
        file = file.replace('ơ', 'ơ');
        file = file.replace('ờ', 'ơ');
        file = file.replace('ớ', 'ơ');
        file = file.replace('ở', 'ơ');
        file = file.replace('ỡ', 'ơ');
        file = file.replace('ợ', 'ơ');
        file = file.replace('Ò', 'o');
        file = file.replace('Ó', 'o');
        file = file.replace('Ỏ', 'o');
        file = file.replace('Õ', 'o');
        file = file.replace('Ọ', 'o');
        file = file.replace('Ô', 'ô');
        file = file.replace('Ồ', 'ô');
        file = file.replace('Ố', 'ô');
        file = file.replace('Ổ', 'ô');
        file = file.replace('Ỗ', 'ô');
        file = file.replace('Ộ', 'ô');
        file = file.replace('Ơ', 'ơ');
        file = file.replace('Ờ', 'ơ');
        file = file.replace('Ớ', 'ơ');
        file = file.replace('Ở', 'ơ');
        file = file.replace('Ỡ', 'ơ');
        file = file.replace('Ợ', 'ơ');
        file = file.replace('ù', 'u');
        file = file.replace('ú', 'u');
        file = file.replace('ủ', 'u');
        file = file.replace('ũ', 'u');
        file = file.replace('ụ', 'u');
        file = file.replace('ư', 'ư');
        file = file.replace('ừ', 'ư');
        file = file.replace('ứ', 'ư');
        file = file.replace('ử', 'ư');
        file = file.replace('ữ', 'ư');
        file = file.replace('ự', 'ư');
        file = file.replace('Ù', 'u');
        file = file.replace('Ú', 'u');
        file = file.replace('Ủ', 'u');
        file = file.replace('Ũ', 'u');
        file = file.replace('Ụ', 'u');
        file = file.replace('Ư', 'ư');
        file = file.replace('Ừ', 'ư');
        file = file.replace('Ứ', 'ư');
        file = file.replace('Ử', 'ư');
        file = file.replace('Ữ', 'ư');
        file = file.replace('Ự', 'ư');
        file = file.replace('ỳ', 'y');
        file = file.replace('ý', 'y');
        file = file.replace('ỷ', 'y');
        file = file.replace('ỹ', 'y');
        file = file.replace('ỵ', 'y');
        file = file.replace('Y', 'y');
        file = file.replace('Ỳ', 'y');
        file = file.replace('Ý', 'y');
        file = file.replace('Ỷ', 'y');
        file = file.replace('Ỹ', 'y');
        file = file.replace('Ỵ', 'y');
        return file;
    }

    /**
     * Generate a random string
     *
     * @param length The length of the random string
     * @return A random string
     */
    public static String createRandomString(int length) {
        if (length <= 0) {
            return "";
        }
        String pattern = "abcdefghijklmnopqrstuvwxyzABCDEFGHIGKLMNOPQRSTUVWXYZ0123456789";
        int count = 0;
        int parrterLen = pattern.length();
        StringBuilder builder = new StringBuilder();
        Random rand = new Random();
        while (count < length) {
            builder.append((pattern.charAt(rand.nextInt(parrterLen))));
            count++;
        }
        return builder.toString();

    }

    /**
     * <p>
     * Create a random string that specify the min length and the max length
     * </p>
     * <p>
     * Author: dungnv93, Created time: 30/06/2017
     * </p>
     *
     * @param minLen
     * @param maxLen
     * @return
     */
    public static String createRandomString(int minLen, int maxLen) {
        if (minLen <= 0) {
            return "";
        }
        Random rand = new Random();
        int newStringLen = rand.nextInt((maxLen - minLen) + 1) + minLen;
        char[] newString = new char[newStringLen];
        for (int i = 0; i < newStringLen; i++) {
            newString[i] = VALID_CHARACTER.charAt(rand.nextInt(VALID_CHARACTER.length()));
        }
        return new String(newString);
    }

    public static String standardInvoiceString(Long input) {
        String temp;
        if (input == null) {
            return "";
        }
        temp = input.toString();
        if (temp.length() <= INVOICE_MAX_LENGTH) {
            int count = INVOICE_MAX_LENGTH - temp.length();
            for (int i = 0; i < count; i++) {
                temp = ZERO + temp;
            }
        }
        return temp;
    }

    public static boolean validString(String temp) {
        return !(temp == null || temp.trim().isEmpty());
    }

    /**
     * validate IP Address
     *
     * @param ip
     * @return
     */
    public static boolean validateIPAddress(String ip) {
        if (isNullOrEmpty(ip)) {
            return false;
        }
        /*String ipAddressPartern
         = "^([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\."
         + "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\."
         + "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\."
         + "([01]?\\d\\d?|2[0-4]\\d|25[0-5])$";
         Pattern pattern = Pattern.compile(ipAddressPartern);*/
        Matcher matcher = IP_ADDRESS_PATTERN.matcher(ip);
        return matcher.matches();
    }

    /**
     * validate multi IP Address
     *
     * @param ipAddress
     * @return
     */
    public static boolean validateMultiIPAddress(String ipAddress) {
        if (isNullOrEmpty(ipAddress)) {
            return false;
        }
        //System.out.println(ipAddress);
        String[] ipArray = ipAddress.split("\\|");

        logger.info("=====> ip " + ipArray.length);
        for (int i = 0, size = ipArray.length; i < size; i++) {
            Matcher matcher = IP_ADDRESS_PATTERN.matcher(ipArray[i].trim());
            if (!matcher.matches()) {
                logger.info("=====> ip " + ipArray[i]);
                return false;
            }
        }
        return true;
    }

    /**
     * getLimitedString check max length
     *
     * @param str
     * @param maxlength
     * @return
     */
    public static String getLimitedString(String str, Integer maxlength) {
        if (str != null && !str.isEmpty()) {
            if (str.length() <= maxlength) {
                return str;
            } else {
                return str.substring(0, maxlength);
            }
        }
        return "";
    }

    /**
     * <p>
     * Last update user: dungnv93
     * </p>
     * <p>
     * Last update Time : 07/09/2017
     * </p>
     * check null or empty
     *
     * @param object
     * @return
     */
    public static Boolean isNullOrEmpty(Object object) {
        if (object == null || object.toString().trim().isEmpty()) {
            return true;
        }
        return false;
    }

    public static boolean isNullOrEmpty(String str) {
        return str == null || str.isEmpty();
    }

    public static boolean equals(String str1, String str2) {
        if (str1 == null) {
            return str2 == null;
        } else if (str2 == null) {
            return false;
        }
        return str1.trim().equals(str2.trim());
    }

    //Dungnv93 - 12/10/2017
    public static boolean equalsConsiderNullIsEmpty(String str1, String str2) {
        str1 = str1 == null ? "" : str1;
        str2 = str2 == null ? "" : str2;

        return str1.trim().equals(str2.trim());
    }

    /**
     * checkRegularExpression in list accept
     *
     * @param input
     * @return
     */
    public static boolean checkRegularExpression(String input) {
        if (isNullOrEmpty(input)) {
            return false;
        }
//        return input.matches("^[A-Za-z0-9 _]+$");
        return input.matches("^[A-Za-z0-9_]+$");
    }

    /**
     * checkSpecialCharacter in list special character
     *
     * @param input
     * @return
     */
    public static boolean checkSpecialCharacter(String input) {
        if (isNullOrEmpty(input)) {
            return false;
        }
        return input.matches(SPECIAL_CHARACTER_PATTERN);
    }

    /**
     * checkInvoiceNo by format
     *
     * @param invoiceNo
     * @return
     */
//    public static boolean checkInvoiceNo(String invoiceNo) {
//        if (isNullOrEmpty((invoiceNo))) {
//            return false;
//        }
//        return invoiceNo.matches("[a-zA-Z0-9/]+");
//    }
    //haint844 updated 20170914
    public static boolean checkInvoiceNo(String invoiceNo) {
        if (isNullOrEmpty((invoiceNo))) {
            return false;
        }
        if (invoiceNo.length() != 13) {
            return false;
        }
        if (invoiceNo.split("/").length != 2) {
            return false;
        }
        if (!isAlpha(invoiceNo.split("/")[0])) {
            return false;
        }
        if (!"E".equalsIgnoreCase(invoiceNo.substring(5, 6))) {
            return false;
        }
        if (!invoiceNo.substring(6).matches("[0-9]+")) {
            return false;
        }
        return invoiceNo.matches("[a-zA-Z0-9/]+");
    }

    /**
     * checkSpecialCharacter1
     *
     * @param value
     * @return
     */
    public static boolean checkSpecialCharacter1(String value) {
        char c;
        int cint;
        int count = 0;
        int length = value.length();
        for (int i = 0; i < length; i++) {
            c = value.charAt(i);
            cint = (int) c;
            if (cint < 48 || (cint > 57 && cint < 65) || (cint > 90 && cint < 97) || cint > 122) {
                count++;
            }
        }
        return count > 0;
    }

    /**
     * convertToStartDate
     *
     * @param startDate
     * @return
     */
    public static Date convertToStartDate(Date startDate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(startDate);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }

    /**
     * convertToEndDate
     *
     * @param startDate
     * @return
     */
    public static Date convertToEndDate(Date startDate) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(startDate);
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        cal.set(Calendar.MILLISECOND, 999);
        return cal.getTime();
    }

    /**
     * checkIssueDateSearch <= 30 day @
     * <p>
     * <p>
     * param issueDateFrom @param issueDateTo @return @param issueDateFrom
     *
     * @param issueDateFrom @param issueDateTo @return
     */
    public static boolean checkIssueDateSearch(Date issueDateFrom, Date issueDateTo) {
        long diff = issueDateTo.getTime() - issueDateFrom.getTime();
        long diffDays = diff / (24 * 60 * 60 * 1000);
        return diffDays <= 31;
    }

    /**
     * encryptPassword SHA-1
     *
     * @return
     */
//    public static String encryptPassword(String password) {
//        MessageDigest md = null;
//        try {
//            md = MessageDigest.getInstance("SHA-1");
//        } catch (Exception e) {
//            logger.error("Exception", e);
//        }
//        if (md == null) {
//            return "";
//        }
//        try {
//            md.update(password.getBytes("UTF-8"));
//        } catch (Exception e) {
//            logger.error("Exception", e);
//        }
//        byte raw[] = md.digest();
//        //String hash = (new BASE64Encoder()).encode(raw);
//        String hash = Base64.encodeBase64String(raw);
//        return hash;
//    }
    public static String format(BigDecimal dblNumber, String strPattern) {
        DecimalFormat fmt = new DecimalFormat(strPattern);
        return fmt.format(dblNumber);
    }

    public static String getFileNameWithoutExtension(String link) {
        String separator = "/";
        String filename = link.substring(link.lastIndexOf(separator) + 1, link.lastIndexOf('.'));
        return filename;
    }

//    public static void replace(File file, String oldString, String newString) throws IOException {
//        String content = org.apache.commons.io.FileUtils.readFileToString(file);
//        org.apache.commons.io.FileUtils.writeStringToFile(file, content.replaceAll(oldString, newString));
//    }

    public static boolean isValidEmailAddress(String email) {
        //<editor-fold defaultstate="collapsed" desc="OLD CODE">

//        boolean result = true;
//        try {
//            InternetAddress emailAddr = new InternetAddress(email);
//            emailAddr.validate();
//        } catch (AddressException e) {
//            logger.error(e);
//            result = false;
//        }
//        return result;
//</editor-fold>
        if (isNullOrEmpty(email)) {
            return false;
        }
        return email.matches(EMAIL_PATTERN);

    }

    public static boolean isValidPhoneNumber(String phoneNumber) {
        if (StringUtils.isNullOrEmpty(phoneNumber)) {
            return false;
        }
        if (!isNumber(phoneNumber.trim())) {
            return false;
        }
        if (phoneNumber.trim().length() > 12) {
            return false;
        }
        return true;
    }

    public static boolean isNumber(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^[0-9]+$";
        return str.matches(pattern);
    }

    public static boolean isNumber2(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^[-0-9]+$";
        return str.matches(pattern);
    }

    public static boolean isPrice(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^[0-9\\.]+$";
        return str.matches(pattern);
    }

    public static boolean checkProductPrice(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }

        return str.matches(REGEX_PRODUCT_PRICE);
    }

    public static boolean checkPhoneFaxNumber(String str) {
        return isNumber(str);
    }

    public static boolean checkIdNo(String str) {
        return isNumber(str);
    }

    /**
     * <p>
     * Check id number </p>
     * <p>
     * Include identity card number, passport number, motobike driving
     * liscence</p>
     * <p>
     * Author: dungnv93, Created date 27/07/2017 </p>
     *
     * @param str
     * @return
     */
    public static boolean checkIdNumber(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }
        String patter = "^[ĐA-Za-z/0-9–-]+$";
        return str.matches(patter);

    }

    public static boolean checkAddress(String str) {
        if (str == null || "".equalsIgnoreCase(str)) {
            return false;
        }
//        String pattern = "^[0-9a-zA-Z_/ –-]+$";
//        return str.matches(pattern);
        return true;
    }

    public static boolean checkAddressAllowVietnamese(String str) {
        try {
            if (isNullOrEmpty(str)) {
                return false;
            }
//        String pattern = "^[^\\!-&*\\+\\.\\:-\\@\\[-\\^\\`\\{-\\~]*$";
//        String pattern = "^[a-zA-Z0-9.,() &'/_-]+$";
//            ParamBO tmpParamBO = MemoryDataLoader.getParamBO("REGEX_VALIDATE_DATA", "ADDRESS");
//            logger.info("ADDRESS regex: " + tmpParamBO.getName());

//            return renameVietnameseToEnglish(str).matches(tmpParamBO.getName());
            return true;
        } catch (Exception ex) {
            logger.error("have error ", ex);
        }
        return false;
    }

    /**
     * <p>
     * Last update user: huypq15 </p>
     * <p>
     * Last up date : 20/08/2017
     * <p>
     * <p>
     *
     * @param str
     * @return
     */
    public static boolean checkTaxCode(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }

        str = str.trim();
        int length = str.length();

        if (length != 10 && length != 14) {
            return false;
        }
        String pattern = "^[0-9]+$";

        if (length == 10) {
            if (!str.matches(pattern)) {
                return false;
            }

        }

        if (length == 14) {
            String[] arr = str.split("-");
            if (arr.length != 2) {
                return false;
            }
            if (!arr[0].matches(pattern) || !arr[1].matches(pattern)) {
                return false;
            }
        }

        int A1 = Integer.parseInt(str.substring(0, 1)) * 31;
        int A2 = Integer.parseInt(str.substring(1, 2)) * 29;
        int A3 = Integer.parseInt(str.substring(2, 3)) * 23;
        int A4 = Integer.parseInt(str.substring(3, 4)) * 19;
        int A5 = Integer.parseInt(str.substring(4, 5)) * 17;
        int A6 = Integer.parseInt(str.substring(5, 6)) * 13;
        int A7 = Integer.parseInt(str.substring(6, 7)) * 7;
        int A8 = Integer.parseInt(str.substring(7, 8)) * 5;
        int A9 = Integer.parseInt(str.substring(8, 9)) * 3;
        int A = A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9;
        int B = A % 11;
        int C = 10 - B;
        if (C != Integer.parseInt(str.substring(9, 10))) {
            return false;
        }

        return true;
    }

    /**
     * Kiem tra MST cua nguoi mua. Sau nay se cho phep MST cua nhieu nuoc
     *
     * @param str
     * @return
     */
    public static boolean checkBuyerTaxCode(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }

        str = str.trim();
        int length = str.length();

        if (length != 10 && length != 14) {
            return true;
        }
        String pattern = "^[0-9]+$";

        //MST Vietnam
        if (length == 10) {
            if (!str.matches(pattern)) {
                return false;
            }

        }

        if (length == 14) {
            String[] arr = str.split("-");
            if (arr.length != 2) {
                return false;
            }
            if (!arr[0].matches(pattern) || !arr[1].matches(pattern)) {
                return false;
            }
        }

        int A1 = Integer.parseInt(str.substring(0, 1)) * 31;
        int A2 = Integer.parseInt(str.substring(1, 2)) * 29;
        int A3 = Integer.parseInt(str.substring(2, 3)) * 23;
        int A4 = Integer.parseInt(str.substring(3, 4)) * 19;
        int A5 = Integer.parseInt(str.substring(4, 5)) * 17;
        int A6 = Integer.parseInt(str.substring(5, 6)) * 13;
        int A7 = Integer.parseInt(str.substring(6, 7)) * 7;
        int A8 = Integer.parseInt(str.substring(7, 8)) * 5;
        int A9 = Integer.parseInt(str.substring(8, 9)) * 3;
        int A = A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9;
        int B = A % 11;
        int C = 10 - B;
        if (C != Integer.parseInt(str.substring(9, 10))) {
            return false;
        }

        return true;
    }

    public static boolean checkDescriptionAllowVietnamese(String description) {
        if (isNullOrEmpty(description)) {
            return false;
        }
        return true;
//        String pattern = "^[a-zA-Z0-9()., _–-]+$";
//        return renameVietnameseToEnglish(description).matches(pattern);
    }

    public static boolean checkUserNameAllowVietnamese(String username) {
        return checkNameAllowVietnamese(username);
    }

    public static boolean checkAcountFormat(String name) {
        if (isNullOrEmpty(name)) {
            return false;
        }
//        String pattern = "^[^\\!-\\,\\.\\/\\:-\\@\\[-\\^\\`\\{-\\~]+$";
        return true;
//        String pattern = "^[a-zA-Z0-9 _.–-]+$";
//        return name.matches(pattern);
    }

    public static boolean checkBankAccountOwnerAllowVietnamese(String username) {
        return checkNameAllowVietnamese(username);
    }

    public static boolean checkUnitNameAllowVietnamese(String username) {
        return checkNameAllowVietnamese(username);
    }

    public static boolean checkFileNameAllowVietnamese(String fileName) {
        if (isNullOrEmpty(fileName)) {
            return false;
        }
        if (fileName.length() > 200) {
            return false;
        }
        fileName = renameVietnameseToEnglish(fileName);
        String regEx = "^[a-zA-Z0-9\\ _–-]+\\.[a-zA-Z]+$";
        return fileName.matches(regEx);
    }

    public static boolean checkAlphaNumeric(String str) {
        if (isNullOrEmpty((str))) {
            return false;
        }
        return str.matches("[a-zA-Z0-9]+$");
    }

    public static boolean checkProductCode(String str) {
        if (StringUtils.isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^[a-zA-Z0-9_.–-]+$";
        return str.matches(pattern);
    }

    public static boolean checkTemplateCode(String str) {
        if (StringUtils.isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^\\w{5,6}0\\/\\d{3}$";
        return str.matches(pattern);
    }

    public static boolean checkProductName(String str) {
        //String pattern = "^[a-zA-Z0-9 _–-]+$";
        if (StringUtils.isNullOrEmpty(str)) {
            return false;
        }
        return true;
//        return str.matches(pattern);
    }

    public static boolean isNumbericAndDots(String str) {
        if (isNullOrEmpty((str))) {
            return false;
        }
        return str.matches("^[\\d\\.]+$");
    }

    /**
     * <p>
     * Last update dev: dungnv93
     * </p>
     * <p>
     * Last update time: 19/09/2017
     * </p>
     *
     * @param productName
     * @return
     */
    public static boolean checkProductNameAllowVietnamese(String productName) {
        try {
            if (isNullOrEmpty(productName)) {
                return false;
            }

            //        String pattern = "^[^\!-\'\*-\,\.\/\:-\@\[-\^\`\{-\~]+$";
            //        String pattern = "^[a-zA-Z0-9 ()_-]+$";
//            ParamBO tmpParamBO = MemoryDataLoader.getParamBO("REGEX_VALIDATE_DATA", "PRODUCT_NAME");
//            logger.info("product name regex: " + tmpParamBO.getName());
//            String productNameAscii = renameVietnameseToEnglish(productName);
//            return productNameAscii.matches(tmpParamBO.getName());
            return true;
        } catch (Exception ex) {
            logger.error("Have error checkProductNameAllowVietnamese ", ex);
        }
        return false;
    }

    /**
     * Check if name is alpha,number, -, _ or space. Especially name can be
     * Vietnamese name
     *
     * @param name
     * @return
     */
    public static boolean checkNameAllowVietnamese(String name) {
        if (isNullOrEmpty(name)) {
            return false;
        }
//        String pattern = "^[^\\!-\\,\\.\\/\\:-\\@\\[-\\^\\`\\{-\\~]+$";
        return true;
//        String pattern = "^[a-zA-Z0-9 _–-]+$";
//        String vnName = renameVietnameseToEnglish(name);
//        return vnName.matches(pattern);
    }

    public static boolean checkCustomerNameAllowVienamese(String str) {
        return checkNameAllowVietnamese(str);
    }

    public static boolean checkEnterpriseNameAllowVietnamese(String name) {
        try {
            if (isNullOrEmpty(name)) {
                return false;
            }
            return true;
//            ParamBO tmpParamBO = MemoryDataLoader.getParamBO("REGEX_VALIDATE_DATA", "ENTERPRISE_NAME");
//            logger.info("ENTERPRISE name regex: " + tmpParamBO.getName());

//        String pattern = "^[a-zA-Z0-9.,\\( \\) &_-]+$";
//        String pattern = "^[a-zA-Z0-9.,() &_-]+$";
//            return renameVietnameseToEnglish(name).matches(tmpParamBO.getName());
        } catch (Exception ex) {
            logger.error("have error ", ex);
        }
        return false;
    }

    public static boolean checkBankNameAllowVietnamese(String str) {
        return checkNameAllowVietnamese(str);
    }

    public static boolean checkDiscount(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^[0-9\\,\\.E]+$";
        return str.matches(pattern);
    }

    public static boolean checkBankAccount(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^[A-Z\\d]+$";
        return str.matches(pattern);
    }

    public static boolean checkWebsite(String str) {
        if (isNullOrEmpty(str)) {
            return false;
        }
        String pattern = "^(https?:\\/\\/)?([\\w\\d\\-_]{2,}\\.+[A-Za-z\\-_]{2,})+\\/?(.)*";

        return str.matches(pattern);
    }

    public static boolean checkLinuxPath(String path) {
        if (isNullOrEmpty(path)) {
            return false;
        }
        String pattern = "^[A-Za-z0-9\\/\\_\\-]*$";

        return path.matches(pattern);

    }

    /**
     * <p>
     * Author: dungnv93
     * </p>
     * <p>
     * Created date: 09/08/2017
     * </p>
     * <p>
     * Code must have one character '_'. The part before it is in the seven
     * invoice group template, the one after must be a chain of numbers.
     * </p>
     *
     * @param code
     * @return
     */
    public static boolean checkInvoiceTemplatePrototypeCode(String code) {
        if (isNullOrEmpty(code)) {
            return false;
        }

        int _pos = code.indexOf("_");//position of the first character '_'
        if (_pos == -1) {
            return false;
        }
        String codePrefix = code.substring(0, _pos);
        String codeSuffix = code.substring(_pos + 1, code.length());

        String[] arrPrefix = {"06HDXK0", "07KPTQ0", "03XKNB0", "04HGDL0", "01GTKT0", "02GTTT0", "08HDDN0", "01BLP0"};
        List<String> listPrefix = Arrays.asList(arrPrefix);

        return listPrefix.contains(codePrefix) && isNumber(codeSuffix);

    }

//    /**
//     * <p>
//     * Author: dungnv93
//     * </p>
//     * <p>
//     * Created date: 10/08/2017
//     * </p>
//     * <p>
//     *
//     * @param listParamBO
//     * @param buyerIdType
//     * @return
//     */
//    public static boolean checkBuyerIdType(List<ParamBO> listParamBO, String buyerIdType) {
//        try {
//            if (listParamBO == null || listParamBO.isEmpty() || isNullOrEmpty(buyerIdType)) {
//                return false;
//            }
//            for (ParamBO paramBO : listParamBO) {
//                if (paramBO != null && buyerIdType.equals(paramBO.getCode())) {
//                    return true;
//                }
//            }
//            return false;
//
//        } catch (Exception ex) {
//            logger.error("Have error checkBuyerIdType", ex);
//        }
//        return false;
//    }

//    /**
//     * <p>
//     * Author: dungnv93
//     * </p>
//     * <p>
//     * Created date: 02/09/2017
//     * </p>
//     * <p>
//     *
//     * @param listParamBO
//     * @param vat
//     * @return
//     */
//    public static boolean checkVat(List<ParamBO> listParamBO, Double vat) {
//        try {
//            if (listParamBO == null || listParamBO.isEmpty() || vat == null) {
//                return false;
//            }
//            for (ParamBO paramBO : listParamBO) {
//                if (paramBO != null && vat.equals(Double.valueOf(paramBO.getName()))) {
//                    return true;
//                }
//            }
//            return false;
//
//        } catch (Exception ex) {
//            logger.error("Have error checkBuyerIdType", ex);
//        }
//        return false;
//    }
//
//    public static boolean checkSmsHeaderFooter(String text) {
//        if (isNullOrEmpty(text)) {
//            return false;
//        }
//        String pattern = "^[a-zA-Z0-9 \\(\\)\\+\\_\\-,'\\.\\/\\:]*$";
//        return text.matches(pattern);
//    }
//
//    /**
//     * <p>
//     * Author: dungnv93 - created date: 24/08/2017 </p>
//     * <p>
//     * View object in json, logging purpose </p>
//     *
//     * @param obj
//     * @return
//     */
//    public static String viewObjectInJson(Object obj) {
//        try {
//            ObjectMapper mapper = new ObjectMapper();
//            String objectInJson = mapper.writeValueAsString(obj);
//            logger.info("Object in json: " + objectInJson);
//            return objectInJson;
//        } catch (JsonProcessingException ex) {
//            logger.error("viewObject have error: ", ex);
//        }
//        return "";
//    }

//    /**
//     * <p>
//     * Author: dungnv93 - created date: 27/08/2017 </p>
//     * <p>
//     * Convert number to thousand format string </p>
//     *
//     * @param number
//     * @return
//     */
//    public static String formatNumberForThousand(Double number) {
//        try {
//            Locale locale = (Locale) FacesContext.getCurrentInstance().getViewRoot().getLocale();
//            logger.info("Locale: " + ((locale == null) ? null : locale.toLanguageTag()));
//            if (locale == null) {
//                //            locale = Locale.ENGLISH;
//                locale = new Locale("vi", "VN");
//            }
//            return NumberFormat.getNumberInstance(locale).format(number);
//        } catch (Exception ex) {
//            logger.error("have error formatNumberForThounsand", ex);
//        }
//        return String.valueOf(number);
//    }

//    public static String autoGenProductCode(String name) {
//        if (StringUtils.isNullOrEmpty(name)) {
//            return name;
//        }
//        String autoGenCode = StringUtils.renameVietnameseToEnglish(name).replace(" ", "_").replaceAll("[^a-zA-Z0-9_.–-]", "");
//        if (autoGenCode.length() > 50) { //50 - maxLength of productCode
//            autoGenCode = autoGenCode.substring(0, 50);
//        }
//        return autoGenCode;
//    }

    //    public int compareVietnamese(String s1, String s2, String[] alphabet) {
//        if (isNullOrEmpty(s2)) {
//            return isNullOrEmpty(s1) ? 0 : 1;
//        }
//
//        if (isNullOrEmpty(s1)) {
//            return isNullOrEmpty(s2) ? 0 : -1;
//        }
//
//        HashMap hashMap = new HashMap();
//        int alphabetLen = alphabet.length;
//        for (int i = 0; i < alphabetLen; i++) {
//            hashMap.put(alphabet[i], i);
//        }
//
//        int input1Len = s1.length();
//        int input2Len = s2.length();
//
//    }
    public static void main(String[] args) {
        String value = "07KPTQ0_011";
        System.out.println(checkTemplateCode(value));
    }

}
